CREATE OR REPLACE PROCEDURE VALIDACOES_RNF(P_CDFILIAL IN VARCHAR,
                                           P_CDOPERADOR IN VARCHAR,
                                           P_DTINICIAL IN DATE,
                                           P_DTFINAL IN DATE)
AS
   V_COUNT NUMBER;
   V_SEQUENCIAL VARCHAR2(20);
   V_NRVALIDACAORNF NUMBER;
   V_DSVALIDACAO VARCHAR2(300);
   
   PROCEDURE INSEREVALIDACAO(P_VALIDACAO IN VARCHAR)
   AS
   BEGIN
      NOVO_CODIGO_NT('VALIDACAORNF', V_SEQUENCIAL);
      V_NRVALIDACAORNF := TO_NUMBER(V_SEQUENCIAL);
      INSERT INTO VALIDACAORNF
         (NRVALIDACAORNF, DSVALIDACAO)
      VALUES
         (V_NRVALIDACAORNF, P_VALIDACAO);
   END;
BEGIN
   DELETE FROM VALIDACAORNF;

   FOR C_PERIODO_RNF IN
      (SELECT DISTINCT E.CDEMPRESA ||' - ' || E.NMEMPRESA DSVALIDACAO
         FROM EMPRESA E, FILIAL F,
              (SELECT DISTINCT F.CDFILIAL
                 FROM V_FILIALLOJACAIXA F, FILIOPER FO
                WHERE ((INSTR(P_CDFILIAL, F.CDFILIAL) > 0) OR (P_CDFILIAL = 'T'))
                  AND FO.CDOPERADOR = P_CDOPERADOR
                  AND F.CDFILIAL = FO.CDFILIAL) FO
        WHERE FO.CDFILIAL = F.CDFILIAL
          AND F.CDEMPRESA = E.CDEMPRESA
          AND NOT EXISTS (SELECT 1
                            FROM PERIODORNF P
                           WHERE E.CDEMPRESA = P.CDEMPRESA
                             AND ((P_DTINICIAL BETWEEN P.DTINICIAL AND P.DTFINAL) OR (P_DTFINAL BETWEEN P.DTINICIAL AND P.DTFINAL)))
        ORDER BY 1) LOOP
      V_DSVALIDACAO := 'EMPRESA '||C_PERIODO_RNF.DSVALIDACAO||' FORA DO PERIODO PERMITIDO PARA GERACAO DE RNF.';
      INSEREVALIDACAO(V_DSVALIDACAO);
   END LOOP;

   /* PRODUTO SEM CODIGO EXTERNO */
   FOR PROD_S_CODEXTERNO IN
      (SELECT DISTINCT P.CDARVPROD||' - '||P.NMPRODUTO PRODVD
         FROM PRODUTO P, ITEMVENDA I, VENDA V, (SELECT F.CDFILIAL, F.CDLOJA, F.CDCAIXA
                                                  FROM V_FILIALLOJACAIXA F, FILIOPER FO
                                                 WHERE ((INSTR(P_CDFILIAL, F.CDFILIAL) > 0) OR (P_CDFILIAL = 'T'))
                                                   AND FO.CDOPERADOR = P_CDOPERADOR
                                                   AND F.CDFILIAL = FO.CDFILIAL) FO
        WHERE P.CDPRODUTO = I.CDPRODUTO
          AND I.CDFILIAL = V.CDFILIAL
          AND I.CDCAIXA = V.CDCAIXA
          AND I.NRSEQVENDA = V.NRSEQVENDA
          AND V.IDSITUVENDA = 'O'
          AND V.DTENTRVENDA BETWEEN P_DTINICIAL AND P_DTFINAL
          AND V.CDFILIAL = FO.CDFILIAL
          AND V.CDCAIXA = FO.CDCAIXA
          AND V.CDLOJA = FO.CDLOJA
          AND P.CDPRODINTE IS NULL
          ORDER BY 1) LOOP
      V_DSVALIDACAO := 'PRODUTO '||PROD_S_CODEXTERNO.PRODVD||' SEM CODIGO EXTERNO PARAMETRIZADO.';
      INSEREVALIDACAO(V_DSVALIDACAO);
   END LOOP;

   /* FILIAL SEM CODIGO EXTERNO */
   FOR C_FIL_S_CODEXTERNO IN
      (SELECT DISTINCT FI.CDFILIAL || ' - ' || FI.NMFILIAL AS FILIALVD
         FROM FILIOPER FO, FILIAL FI
        WHERE ((INSTR(P_CDFILIAL, FI.CDFILIAL) > 0) OR (P_CDFILIAL = 'T'))
          AND FI.CDFILIAL = FO.CDFILIAL
          AND FO.CDOPERADOR = P_CDOPERADOR
          AND FI.CDFILIALEXT IS NULL
          AND EXISTS (SELECT 1
                        FROM CAIXA CX, CAIXAPERS CP
                       WHERE CX.CDFILIAL = FI.CDFILIAL
                         AND CX.IDCOLETOR IN ('R', 'N')
                         AND CX.CDFILIAL = CP.CDFILIAL
                         AND CX.CDCAIXA = CP.CDCAIXAPERS)
        ORDER BY 1) LOOP
      V_DSVALIDACAO := 'FILIAL '||C_FIL_S_CODEXTERNO.FILIALVD||' SEM CODIGO EXTERNO PARAMETRIZADO.';
      INSEREVALIDACAO(V_DSVALIDACAO);
   END LOOP;

   /* FILIAL COM CODIGO EXTERNO MAIOR QUE 5 CARACTERES */
   FOR C_COD_EXT_GRANDE IN
      (SELECT DISTINCT FI.CDFILIAL || ' - ' || FI.NMFILIAL AS FILIALVD
         FROM FILIOPER FO, FILIAL FI
        WHERE ((INSTR(P_CDFILIAL, FI.CDFILIAL) > 0) OR (P_CDFILIAL = 'T'))
          AND FI.CDFILIAL = FO.CDFILIAL
          AND FO.CDOPERADOR = P_CDOPERADOR
          AND LENGTH(FI.CDFILIALEXT) > 5
        ORDER BY 1) LOOP
      V_DSVALIDACAO := 'FILIAL '||C_COD_EXT_GRANDE.FILIALVD||' COM CODIGO EXTERNO MAIOR QUE 5 CARACTERES.';
      INSEREVALIDACAO(V_DSVALIDACAO);
   END LOOP;

   /* VALIDA PONTOS NO NCM */
   FOR PONTOS_NCM IN
      (SELECT DISTINCT 'NCM DO PRODUTO '||P.CDARVPROD||' - '||P.NMPRODUTO||' CONTEM PONTOS. O NCM '||
                       'DEVE CONTER APENAS NUMEROS NO CADASTRO.' AS DSVALIDACAO
         FROM ITEMVENDA I, VENDA V, PRODUTO P,
             (SELECT F.CDFILIAL, F.CDLOJA, F.CDCAIXA
                FROM V_FILIALLOJACAIXA F, FILIOPER FO
               WHERE ((INSTR(P_CDFILIAL, F.CDFILIAL) > 0) OR (P_CDFILIAL = 'T'))
                 AND FO.CDOPERADOR = P_CDOPERADOR
                 AND F.CDFILIAL = FO.CDFILIAL) FO
        WHERE P.CDPRODUTO = I.CDPRODUTO
          AND I.CDFILIAL = V.CDFILIAL
          AND I.CDCAIXA = V.CDCAIXA
          AND I.NRSEQVENDA = V.NRSEQVENDA
          AND I.CDPRODUTO = P.CDPRODUTO
          AND V.IDSITUVENDA = 'O'
          AND V.DTENTRVENDA BETWEEN P_DTINICIAL AND P_DTFINAL
          AND V.CDFILIAL = FO.CDFILIAL
          AND V.CDCAIXA = FO.CDCAIXA
          AND V.CDLOJA = FO.CDLOJA
          AND P.CDCLASFISC IS NOT NULL
          AND INSTR(P.CDCLASFISC, '.') > 0
        ORDER BY 1) LOOP
      INSEREVALIDACAO(PONTOS_NCM.DSVALIDACAO);
   END LOOP;

   /* VALIDA TAMANHO DO NCM */
   FOR TAM_NCM IN
      (SELECT DISTINCT 'NCM DO PRODUTO '||P.CDARVPROD||' - '||P.NMPRODUTO||' POSSUI TAMANHO INCORRETO.' AS DSVALIDACAO
         FROM ITEMVENDA I, VENDA V, PRODUTO P,
             (SELECT F.CDFILIAL, F.CDLOJA, F.CDCAIXA
                FROM V_FILIALLOJACAIXA F, FILIOPER FO
               WHERE ((INSTR(P_CDFILIAL, F.CDFILIAL) > 0) OR (P_CDFILIAL = 'T'))
                 AND FO.CDOPERADOR = P_CDOPERADOR
                 AND F.CDFILIAL = FO.CDFILIAL) FO
        WHERE P.CDPRODUTO = I.CDPRODUTO
          AND I.CDFILIAL = V.CDFILIAL
          AND I.CDCAIXA = V.CDCAIXA
          AND I.NRSEQVENDA = V.NRSEQVENDA
          AND I.CDPRODUTO = P.CDPRODUTO
          AND V.IDSITUVENDA = 'O'
          AND V.DTENTRVENDA BETWEEN P_DTINICIAL AND P_DTFINAL
          AND V.CDFILIAL = FO.CDFILIAL
          AND V.CDCAIXA = FO.CDCAIXA
          AND V.CDLOJA = FO.CDLOJA
          AND (P.CDCLASFISC IS NULL OR LENGTH(P.CDCLASFISC) <> 8)
        ORDER BY 1) LOOP
      INSEREVALIDACAO(TAM_NCM.DSVALIDACAO);
   END LOOP;

   /* TIPO DE DOCUMENTO NAO CADASTRADO PARA A CHAVE CFOP + CST + NCM */
   FOR C_TIPODOC IN
      (SELECT DISTINCT 'TIPO DE DOCUMENTO NAO CADASTRADO PARA O CFOP '||II.CDCFOPPROD||', NCM '||P.CDCLASFISC||
                       ' E CST (ICMS) '||II.CDCSTPRODI||'.' AS DSVALIDACAO
         FROM ITEMVENDA I, VENDA V, ITVENDAIMPOS II, PRODUTO P,
             (SELECT F.CDFILIAL, F.CDLOJA, F.CDCAIXA
                FROM V_FILIALLOJACAIXA F, FILIOPER FO
               WHERE ((INSTR(P_CDFILIAL, F.CDFILIAL) > 0) OR (P_CDFILIAL = 'T'))
                 AND FO.CDOPERADOR = P_CDOPERADOR
                 AND F.CDFILIAL = FO.CDFILIAL) FO
        WHERE P.CDPRODUTO = I.CDPRODUTO
          AND I.CDFILIAL = V.CDFILIAL
          AND I.CDCAIXA = V.CDCAIXA
          AND I.NRSEQVENDA = V.NRSEQVENDA
          AND I.CDFILIAL = II.CDFILIAL
          AND I.CDCAIXA = II.CDCAIXA
          AND I.NRSEQVENDA = II.NRSEQVENDA
          AND I.NRSEQUITVEND = II.NRSEQUITVEND
          AND I.CDPRODUTO = P.CDPRODUTO
          AND V.IDSITUVENDA = 'O'
          AND V.DTENTRVENDA BETWEEN P_DTINICIAL AND P_DTFINAL
          AND V.CDFILIAL = FO.CDFILIAL
          AND V.CDCAIXA = FO.CDCAIXA
          AND V.CDLOJA = FO.CDLOJA
          AND NOT EXISTS (SELECT 1
                            FROM TPDOCINTSAPFOS T
                           WHERE T.CDCFOP = II.CDCFOPPROD
                             AND T.CDCST = II.CDCSTPRODI
                             AND T.CDNCM = P.CDCLASFISC)
        ORDER BY 1) LOOP
      INSEREVALIDACAO(C_TIPODOC.DSVALIDACAO);
   END LOOP;

   /* CODIGO EXTERNO DA EMPRESA */
   FOR C_EMPRESA IN
      (SELECT E.CDEMPRESA || E.NMEMPRESA DSVALIDACAO
         FROM EMPRESA E, FILIAL F,
              (SELECT DISTINCT F.CDFILIAL
                 FROM V_FILIALLOJACAIXA F, FILIOPER FO
                WHERE ((INSTR(P_CDFILIAL, F.CDFILIAL) > 0) OR (P_CDFILIAL = 'T'))
                  AND FO.CDOPERADOR = P_CDOPERADOR
                  AND F.CDFILIAL = FO.CDFILIAL) FO
        WHERE FO.CDFILIAL = F.CDFILIAL
          AND F.CDEMPRESA = E.CDEMPRESA
          AND E.CDEXTERNO IS NULL
        ORDER BY 1) LOOP
      V_DSVALIDACAO := 'EMPRESA '||C_EMPRESA.DSVALIDACAO||' SEM CODIGO EXTERNO PARAMETRIZADO.';
      INSEREVALIDACAO(V_DSVALIDACAO);
   END LOOP;

   /* CODIGO DE CONVERSAO DA FILIAL */
   FOR C_FIL_CONV IN
      (SELECT CDFILIAL, NMFILIAL
         FROM (SELECT F.CDFILIAL, F.NMFILIAL, F.CDFILIALEXT
                 FROM FILIAL F, (SELECT DISTINCT F.CDFILIAL
                                   FROM V_FILIALLOJACAIXA F, FILIOPER FO
                                  WHERE ((INSTR(P_CDFILIAL, F.CDFILIAL) > 0) OR (P_CDFILIAL = 'T'))
                                    AND FO.CDOPERADOR = P_CDOPERADOR
                                    AND F.CDFILIAL = FO.CDFILIAL) FO
               WHERE F.CDFILIAL = FO.CDFILIAL
                 AND LENGTH(CDFILIALEXT) <= 5
                 AND EXISTS (SELECT 1
                               FROM CAIXA CX, CAIXAPERS CP
                              WHERE CX.CDFILIAL = F.CDFILIAL
                                AND CX.IDCOLETOR IN ('R', 'N')
                                AND CX.CDFILIAL = CP.CDFILIAL
                                AND CX.CDCAIXA = CP.CDCAIXAPERS)) F
        WHERE FILIALSAPRNF(CDFILIAL) IS NULL
        ORDER BY 1) LOOP
      V_DSVALIDACAO := 'FILIAL '||C_FIL_CONV.CDFILIAL||' - '||C_FIL_CONV.NMFILIAL||' SEM CODIGO DE CONVERSAO PARAMETRIZADO.';
      INSEREVALIDACAO(V_DSVALIDACAO);
   END LOOP;

   /* CLIENTE PADRAO */
   FOR C_CLIENTEPADRAO IN
      (SELECT FO.CDFILIAL, FO.NMFILIAL
         FROM (SELECT DISTINCT F.CDFILIAL, F.NMFILIAL
                 FROM V_FILIALLOJACAIXA F, FILIOPER FO
                WHERE ((INSTR(P_CDFILIAL, F.CDFILIAL) > 0) OR (P_CDFILIAL = 'T'))
                  AND FO.CDOPERADOR = P_CDOPERADOR
                  AND F.CDFILIAL = FO.CDFILIAL) FO, PARAVEND P
        WHERE P.CDFILIAL = FO.CDFILIAL
          AND P.CDCLIENTE IS NULL
          AND EXISTS (SELECT 1
                        FROM CAIXA CX, CAIXAPERS CP
                       WHERE CX.CDFILIAL = FO.CDFILIAL
                         AND CX.IDCOLETOR IN ('R', 'N')
                         AND CX.CDFILIAL = CP.CDFILIAL
                         AND CX.CDCAIXA = CP.CDCAIXAPERS)
        ORDER BY 1) LOOP
      V_DSVALIDACAO := 'FILIAL '||C_CLIENTEPADRAO.CDFILIAL||' - '||C_CLIENTEPADRAO.NMFILIAL||' SEM CLIENTE PADRAO PARAMETRIZADO.';
      INSEREVALIDACAO(V_DSVALIDACAO);
   END LOOP;

   /* CODIGO EXTERNO DO CLIENTE PADRAO */
   FOR C_CODEXTCLIPADRAO IN
      (SELECT FO.CDFILIAL, CL.CDCLIENTE, CL.NMRAZSOCCLIE
         FROM (SELECT DISTINCT F.CDFILIAL, F.NMFILIAL
                 FROM V_FILIALLOJACAIXA F, FILIOPER FO
                WHERE ((INSTR(P_CDFILIAL, F.CDFILIAL) > 0) OR (P_CDFILIAL = 'T'))
                  AND FO.CDOPERADOR = P_CDOPERADOR
                  AND F.CDFILIAL = FO.CDFILIAL) FO, PARAVEND P LEFT JOIN CLIENTE CL
                                                                      ON P.CDCLIENTE = CL.CDCLIENTE
        WHERE FO.CDFILIAL = P.CDFILIAL
          AND CL.CDEXCLIENTE IS NULL
          AND EXISTS (SELECT 1
                        FROM CAIXA CX, CAIXAPERS CP
                       WHERE CX.CDFILIAL = FO.CDFILIAL
                         AND CX.IDCOLETOR IN ('R', 'N')
                         AND CX.CDFILIAL = CP.CDFILIAL
                         AND CX.CDCAIXA = CP.CDCAIXAPERS)
        ORDER BY 1) LOOP
      V_DSVALIDACAO := 'CLIENTE PADRAO ('||C_CODEXTCLIPADRAO.CDCLIENTE||' - '||C_CODEXTCLIPADRAO.NMRAZSOCCLIE||')'||
                       ' RELACIONADO A FILIAL '||C_CODEXTCLIPADRAO.CDFILIAL||' NAO POSSUI CODIGO EXTERNO PARAMETRIZADO.';
      INSEREVALIDACAO(V_DSVALIDACAO);
   END LOOP;

   /* RETIRADO A PEDIDO DA SODEXO
   FOR C_TIPOEMISDIF IN
      (SELECT DISTINCT FO.CDFILIAL, FO.NMFILIAL
         FROM CAIXA C, (SELECT DISTINCT F.CDFILIAL, F.NMFILIAL
                          FROM V_FILIALLOJACAIXA F, FILIOPER FO
                         WHERE ((INSTR(P_CDFILIAL, F.CDFILIAL) > 0) OR (P_CDFILIAL = 'T'))
                           AND FO.CDOPERADOR = P_CDOPERADOR
                           AND F.CDFILIAL = FO.CDFILIAL) FO, CAIXAPERS CP
        WHERE C.CDFILIAL = FO.CDFILIAL
          AND C.IDCOLETOR IN ('R', 'N')
          AND C.CDFILIAL = CP.CDFILIAL
          AND C.CDCAIXA = CP.CDCAIXAPERS
          AND EXISTS (SELECT 1
                        FROM CAIXA CX, CAIXAPERS CPS
                       WHERE CX.CDFILIAL = C.CDFILIAL
                         AND CX.IDTPEMISSAOFOS <> C.IDTPEMISSAOFOS
                         AND CX.IDCOLETOR IN ('R', 'N')
                         AND CX.CDFILIAL = CPS.CDFILIAL
                         AND CX.CDCAIXA = CPS.CDCAIXAPERS)
        ORDER BY 1) LOOP
      V_DSVALIDACAO := 'A FILIAL '||C_TIPOEMISDIF.CDFILIAL||' - '||C_TIPOEMISDIF.NMFILIAL||
                       ' POSSUI CAIXAS COM MAIS DE UM TIPO DE EMISSAO. FAVOR CORRIGIR ESSA SITUACAO.';
      INSEREVALIDACAO(V_DSVALIDACAO);
   END LOOP;
   */

   FOR C_TAM_EMPRESA IN
      (SELECT E.CDEMPRESA, E.NMEMPRESA
         FROM EMPRESA E, FILIAL F,
              (SELECT DISTINCT F.CDFILIAL, F.NMFILIAL
                 FROM V_FILIALLOJACAIXA F, FILIOPER FO
                WHERE ((INSTR(P_CDFILIAL, F.CDFILIAL) > 0) OR (P_CDFILIAL = 'T'))
                  AND FO.CDOPERADOR = P_CDOPERADOR
                  AND F.CDFILIAL = FO.CDFILIAL) FO
        WHERE F.CDFILIAL = FO.CDFILIAL
          AND F.CDEMPRESA = E.CDEMPRESA
          AND LENGTH(E.CDEXTERNO) > 4) LOOP
      V_DSVALIDACAO := 'CODIGO EXTERNO DA EMPRESA '||C_TAM_EMPRESA.CDEMPRESA||' - ' ||C_TAM_EMPRESA.NMEMPRESA||
                       ' MAIOR QUE 4 CARACTERES.';
      INSEREVALIDACAO(V_DSVALIDACAO);
   END LOOP;

   /* VERIFICA ULTIMO FECHAMENTO DO DIA FEITO */
   FOR C_FECH IN
      (SELECT T.CDFILIAL, T.CDCAIXA, TO_CHAR(DTABERCAIX, 'DD/MM/YYYY') DIA, MAX(DTABERCAIX) DTABERCAIX
         FROM TURCAIXA T, (SELECT DISTINCT F.CDFILIAL, F.NMFILIAL
                             FROM V_FILIALLOJACAIXA F, FILIOPER FO
                            WHERE ((INSTR(P_CDFILIAL, F.CDFILIAL) > 0) OR (P_CDFILIAL = 'T'))
                              AND FO.CDOPERADOR = P_CDOPERADOR
                              AND F.CDFILIAL = FO.CDFILIAL) FO, CAIXA C, CAIXAPERS CP
        WHERE T.CDFILIAL = FO.CDFILIAL
          AND T.CDFILIAL = C.CDFILIAL
          AND T.CDCAIXA = C.CDCAIXA
          AND C.IDCOLETOR IN ('N', 'R')
          AND C.CDFILIAL = CP.CDFILIAL
          AND C.CDCAIXA = CP.CDCAIXAPERS
          AND TRUNC(DTABERCAIX) BETWEEN P_DTINICIAL AND P_DTFINAL
        GROUP BY T.CDFILIAL, T.CDCAIXA, TO_CHAR(DTABERCAIX, 'DD/MM/YYYY')
        ORDER BY 1, 2, 4) LOOP
      V_COUNT := 0;
      BEGIN
         SELECT COUNT(*)
           INTO V_COUNT
           FROM TURCAIXA T, CAIXA C
          WHERE T.CDFILIAL = C_FECH.CDFILIAL
            AND TRUNC(T.DTABERCAIX) = TO_DATE(C_FECH.DIA, 'DD/MM/YYYY')
            AND T.IDULTFECHAMEN = 'S'
            AND T.CDFILIAL = C.CDFILIAL
            AND T.CDCAIXA = C.CDCAIXA
            AND C.IDCOLETOR IN ('N', 'R');
      EXCEPTION
         WHEN NO_DATA_FOUND THEN
            V_COUNT := 0;
      END;
      IF V_COUNT = 0 THEN
         FOR C_VDFECH IN
            (SELECT 1
               FROM TURCAIXA
              WHERE CDFILIAL = C_FECH.CDFILIAL
                AND CDCAIXA = C_FECH.CDCAIXA
                AND DTABERCAIX = C_FECH.DTABERCAIX
                AND IDULTFECHAMEN <> 'S') LOOP
            V_DSVALIDACAO := 'CAIXA '||C_FECH.CDCAIXA||' DA FILIAL '||C_FECH.CDFILIAL||' NAO POSSUI O ULTIMO FECHAMENTO DO DIA '||C_FECH.DIA;
            INSEREVALIDACAO(V_DSVALIDACAO);
         END LOOP;
      END IF;
   END LOOP;
END;
/

-- VERSAO --
BEGIN ATUVEROBJ('VALIDACOES_RNF','6.49.055'); END;
/
