/* Sistemas Integrados TEKNISA
   Gesto Consolidada

Nome: 0012_CVISTPO_ETL_CONTABIL.sql

Descrio:                                                                 
Procedure para preencher os fatos da tabela ETLCONTABIL a partir dos dados
apurados dos lanamentos contbeis.

Criao:                                                                   
10/06/2008 Leonardo Bispo

*/

CREATE OR REPLACE
PROCEDURE ETL_CONTABIL
                     (P_DTINI  IN DATE,
                      P_DTFINA IN DATE)

IS
  V_DTFIM           DATE;
  V_DTINI           DATE;
  V_NRSEQFATCON     VARCHAR2(12);
  V_QTLANCAUTOM     FLOAT(126);
  V_QTLANCMANUA     FLOAT(126);
  V_QTLANCALTER     FLOAT(126);
  V_CDOPERINS       VARCHAR2(12);
  V_DTMES           VARCHAR2(07);
  V_DTANO           VARCHAR2(04);
  V_BIM             VARCHAR2(6);
  V_TRI             VARCHAR2(6);
  V_SEM             VARCHAR2(6);
BEGIN
 V_DTINI  := P_DTINI;

    V_DTMES  :=  TO_CHAR(V_DTINI,'MM');
    V_DTANO  :=  TO_CHAR(V_DTINI,'YYYY');

    DELETE FROM ETLCONTABIL
    WHERE  (ANOFATCON > V_DTANO) OR
          (ANOFATCON = V_DTANO
          AND  MESFATCON >= V_DTMES);

    BEGIN
       SELECT NVL(MAX(LPAD(NRSEQFATCON,12,0)),0)
       INTO   V_NRSEQFATCON
       FROM   ETLCONTABIL;
    EXCEPTION
    WHEN OTHERS THEN
       V_NRSEQFATCON := '0';
    END;

    WHILE (V_DTINI  <=  P_DTFINA) LOOP

       V_DTFIM := LAST_DAY( V_DTINI);

       FOR C_ETL IN
          (SELECT
              SUM(AUTOM) AS QTLANCAUTOM,
              SUM(MANU) AS QTLANCMANUA,
              SUM(ALTE) AS QTLANCALTER,
              CASE
                 WHEN CDOPERINS IN (SELECT CDOPERADOR FROM OPERADOR)  THEN CDOPERINS
              ELSE NULL
              END CDOPERINS,
              CDEMPRESA, CDFILIAL, CDCONTCTBL,
              CDHISTPADR
           FROM
              (
               SELECT
                  COUNT(*) AS AUTOM,
                  0 AS MANU,
                  0 AS ALTE,
                  L.CDEMPRESA, F.CDFILIAL, L.CDCONTCTBL,
                  L.CDOPERINS, L.CDHISTPADR
               FROM
                  LANCCONT L, FILIAL F
               WHERE
                  L.CDEMPRESA = F.CDEMPRESA AND
                  IDLANCINTE = 'I' AND
                  (TRUNC(L.DTLANCCONT)  BETWEEN V_DTINI AND V_DTFIM)
               GROUP BY
                  L.CDEMPRESA, F.CDFILIAL, L.CDCONTCTBL,
                  L.CDOPERINS, L.CDHISTPADR
               UNION ALL   
               SELECT
                  0 AS AUTOM,
                  COUNT(*) AS MANU,
                  0 AS ALTE,
                  L.CDEMPRESA, F.CDFILIAL, L.CDCONTCTBL,
                  L.CDOPERINS, L.CDHISTPADR
               FROM
                  LANCCONT L, FILIAL F
               WHERE
                  L.CDEMPRESA = F.CDEMPRESA AND
                  IDLANCINTE  <> 'I' AND
                  (TRUNC(L.DTLANCCONT)  BETWEEN V_DTINI AND V_DTFIM)             
               GROUP BY
                  L.CDEMPRESA, F.CDFILIAL, L.CDCONTCTBL,
                  L.CDOPERINS, L.CDHISTPADR
               UNION ALL
               SELECT
                  0 AS AUTOM,
                  0 AS MANU,
                  COUNT(*) AS ALTE,
                  L.CDEMPRESA, F.CDFILIAL, L.CDCONTCTBL,
                  L.CDOPERINS, L.CDHISTPADR
               FROM
                  LANCCONT L, FILIAL F
               WHERE
                  L.CDEMPRESA = F.CDEMPRESA AND
                  IDLANCINTE  = 'I' AND IDLANCMANI = 'S' AND
                  (TRUNC(L.DTLANCCONT)  BETWEEN V_DTINI AND V_DTFIM)
               GROUP BY
                  L.CDEMPRESA, F.CDFILIAL, L.CDCONTCTBL,
                  L.CDOPERINS, L.CDHISTPADR
              )
           GROUP BY
              CDEMPRESA, CDFILIAL, CDCONTCTBL,
              CDOPERINS, CDHISTPADR
          )

       LOOP

             V_DTMES  :=  TO_CHAR(V_DTFIM,'MM');
             V_DTANO  :=  TO_CHAR(V_DTFIM,'YYYY');

             V_BIM := TRUNC ((V_DTMES - 1) / 2) + 1;
             V_TRI := TRUNC ((V_DTMES - 1) / 3) + 1;
             V_SEM := TRUNC ((V_DTMES - 1) / 6) + 1;

             V_NRSEQFATCON := LPAD(V_NRSEQFATCON + 1, 12 ,'0');
              INSERT INTO ETLCONTABIL
                      (NRSEQFATCON, QTLANCAUTOM, QTLANCALTER,
                       QTLANCMANUA, CDFILIAL, CDEMPRESA,
                       CDCONTCTBL, CDOPERINSERE, CDHISTPADR,
                       ANOFATCON, SEMFATCON, TRIFATCON,
                       BIMFATCON, MESFATCON)
              VALUES               
                      (V_NRSEQFATCON, C_ETL.QTLANCAUTOM, C_ETL.QTLANCALTER,
                       C_ETL.QTLANCMANUA, C_ETL.CDFILIAL, C_ETL.CDEMPRESA,
                       C_ETL.CDCONTCTBL, C_ETL.CDOPERINS, C_ETL.CDHISTPADR,
                       V_DTANO, V_SEM||' Sem',  V_TRI||' Tri',
                       V_BIM||' Bim', V_DTMES);
       END LOOP;

    COMMIT;
    V_DTINI := ADD_MONTHS(V_DTINI,1);
    END LOOP;
END;
/

BEGIN
  CRIA_FATOS('ETLCONTABIL', 'QTLANCMANUA', 'Lanamento Manual',                'Contbil');
  CRIA_FATOS('ETLCONTABIL', 'QTLANCAUTOM', 'Lanamento Automtico',            'Contbil');
  CRIA_FATOS('ETLCONTABIL', 'QTLANCALTER', 'Lanamento Automtico Alterado',   'Contbil');

   CRIA_DIME('ETLCONTABIL', 'ANOFATCON',    'Ano',       NULL, NULL, NULL, NULL);
   CRIA_DIME('ETLCONTABIL', 'SEMFATCON',    'Semestre',  NULL, NULL, NULL, NULL);
   CRIA_DIME('ETLCONTABIL', 'TRIFATCON',    'Trimestre', NULL, NULL, NULL, NULL);
   CRIA_DIME('ETLCONTABIL', 'BIMFATCON',    'Bimestre',  NULL, NULL, NULL, NULL);
   CRIA_DIME('ETLCONTABIL', 'MESFATCON',    'Ms',       NULL, NULL, NULL, NULL);
   CRIA_DIME('ETLCONTABIL', 'CDEMPRESA',    'Empresa',  'EMPRESA',  'CDEMPRESA',  'NMEMPRESA', NULL);
   CRIA_DIME('ETLCONTABIL', 'CDFILIAL',     'Filial',   'FILIAL',   'CDFILIAL',   'NMFILIAL', NULL);
   CRIA_DIME('ETLCONTABIL', 'CDOPERINSERE', 'Operador', 'OPERADOR', 'CDOPERADOR', 'NMOPERADOR', NULL);

   CRIA_DIME('ETLCONTABIL', 'CDCONTCTBL', 'Conta Contbil',   'CONTCTBL', 'CDCONTCTBL', 'NMCONTCTBL', 'CDCONTANTE');
   CRIA_DIME('ETLCONTABIL', 'CDHISTPADR', 'Histrico Padro', 'HISTPADR', 'CDHISTPADR', 'DSHISTPADR', NULL);

   COMMIT;
END;
/

DECLARE
   V_CDSEQUENCIAL VARCHAR2(20) NULL;
BEGIN
    /*REGISTRANDO A PROCEDURE PARA O AGENDADOR DE EVENTOS*/
    SELECT COUNT(*)
    INTO   V_CDSEQUENCIAL
    FROM   EVENTOAEV 
    WHERE  NMOBJBDEVEN = 'ETL_CONTABIL';
    
    IF V_CDSEQUENCIAL = 0 THEN
        BEGIN
        
            SELECT LPAD(NVL(MAX(LPAD(CDEVENTOAEV, 10, 0))+1,0),10,'0')
            INTO   V_CDSEQUENCIAL
            FROM   EVENTOAEV;
        
            INSERT INTO EVENTOAEV 
            (CDEVENTOAEV, NMEVENTOAEV, IDTPEVENAEV, CDOPERINCL, DTINCLEVEN, NMOBJBDEVEN)
            VALUES
            (V_CDSEQUENCIAL,'ETL Contbil','STP', NULL, SYSDATE, 'ETL_CONTABIL');
            COMMIT;
            
        EXCEPTION
           WHEN OTHERS THEN
              RAISE_APPLICATION_ERROR (-20001, 'Falha no cadastro de eventos agendveis.');
        END;
    END IF;
END;
/

DECLARE
   V_CDSEQUENCIAL VARCHAR2(20) NULL;
BEGIN
    SELECT COUNT(*)
    INTO   V_CDSEQUENCIAL
    FROM   PARAMEVENTO
    WHERE  CDEVENTOAEV = (SELECT CDEVENTOAEV
                          FROM   EVENTOAEV
                          WHERE  NMOBJBDEVEN = 'ETL_CONTABIL');
 
    IF V_CDSEQUENCIAL = 0 THEN
        BEGIN
        
        SELECT CDEVENTOAEV
        INTO   V_CDSEQUENCIAL
        FROM   EVENTOAEV
        WHERE  NMOBJBDEVEN = 'ETL_CONTABIL';
    
            INSERT INTO PARAMEVENTO
            (CDEVENTOAEV,NMPARAMEVEN,IDDADPARAMEVEN,IDTIPPARAMEVEN,DSPARAMEVEN,IDPARAMNULL,VRTAMPARAM,NRSEQPARAMEVEN) 
            VALUES
            (V_CDSEQUENCIAL,'P_DTINI','D','E','Data Inicial','N',0,'01');
            COMMIT;

            INSERT INTO PARAMEVENTO
            (CDEVENTOAEV,NMPARAMEVEN,IDDADPARAMEVEN,IDTIPPARAMEVEN,DSPARAMEVEN,IDPARAMNULL,VRTAMPARAM,NRSEQPARAMEVEN) 
            VALUES
            (V_CDSEQUENCIAL,'P_DTFINA','D','E','Data Final','N',0,'02');
            COMMIT;
            
        EXCEPTION
           WHEN OTHERS THEN
              RAISE_APPLICATION_ERROR (-20001, 'Falha no cadastro de eventos agendveis.');
        END;
    END IF;
END;
/

-- VERSAO --
BEGIN ATUVEROBJ('ETL_CONTABIL','5.05.020'); END;
/
